// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// dllapi.cpp - implementation of Half-Life DLL routines

#include <extdll.h>
#include <sdk_util.h>
#include <cbase.h>
#include <player.h>

#include <dllapi.h>
#include <meta_api.h>
#include <sdk_util.h>

#include "statsme_api.h"		// me

player_t	players[33];
bool		sm_enabled = true;
announce_t	score;

void ServerActivate_Post( edict_t *pEdictList, int edictCount, int clientMax )
{
	memset(&players,0,sizeof(players));
	memset(&score,0,sizeof(score));

	if (gmsgCurWeapon)
	{
		RETURN_META(MRES_IGNORED);
	}

	gmsgCurWeapon	=	GET_USER_MSG_ID(PLID, "CurWeapon", NULL );
	gmsgDamage		=	GET_USER_MSG_ID(PLID, "Damage", NULL );
	gmsgDeathMsg	=	GET_USER_MSG_ID(PLID, "DeathMsg", NULL );
	gmsgTextMsg		=	GET_USER_MSG_ID(PLID, "TextMsg", NULL );
	gmsgStatusIcon	=	GET_USER_MSG_ID(PLID, "StatusIcon", NULL );
	gmsgTeamInfo	=	GET_USER_MSG_ID(PLID, "TeamInfo", NULL );
	gmsgWeaponList	=	GET_USER_MSG_ID(PLID, "WeaponList", NULL );
	gmsgResetHUD	=   GET_USER_MSG_ID(PLID, "ResetHUD", NULL );
	gmsgMOTD		=	GET_USER_MSG_ID(PLID, "MOTD", NULL );
	gmsgServerName	=	GET_USER_MSG_ID(PLID, "ServerName", NULL );
	gmsgAmmoX 		=	GET_USER_MSG_ID(PLID, "AmmoX", NULL );

	int size; 
	const char *cp;

	for(int i=0; i < MAX_REG_MSGS; i++) 
	{
		cp=GET_USER_MSG_NAME(PLID, i, &size);

		if(cp && FStrEq("intermission?",cp))
			gmsgintermission = i;
	}

	RETURN_META(MRES_IGNORED);
}

void MakeStats(player_t *pPlayer, char* pList)
{
	pList[0] = 0;
	score.accuracy = 0;
	int hits = 0;
	int shots = 0;
				
	for (int c = 0; weapon_select[c].iId ; ++c)
	{
		if (pPlayer->a_weapons[weapon_select[c].iId].bulletsfired) 
		{	
			shots += pPlayer->a_weapons[weapon_select[c].iId].bulletsfired;
			hits += pPlayer->a_weapons[weapon_select[c].iId].bulletshit;
		}
	}
				
	score.accuracy = ACCURACY_FORMULA(shots,hits);
	score.killstodeaths = EFFEC_FORMULA(pPlayer->kills,pPlayer->deaths);
	score.kills = pPlayer->kills;
	score.deaths = pPlayer->deaths;
				
	strcat(pList,UTIL_VarArgs("%s\n",UTIL_FillWithInfo(sm_playerstats->string)));
	
	for (int i = 0; weapon_select[i].iId ; ++i)
	{
		if (pPlayer->a_weapons[weapon_select[i].iId].bulletsfired) 
		{
			//name<userid><wonid><team> "weapon x" "shots x" "hits x" "headshots x"
			strcat(pList,UTIL_VarArgs("%s\n",UTIL_FillWithWeapon(sm_weaponstats->string,
				weapon_select[i].name,pPlayer->a_weapons[weapon_select[i].iId])));
		}
	}
}

void StartFrame_Post( void )
{
	if (sm_enabled)
	{	
		if ((score.mp_c4timer > 0) && SM_MODE(TIME_ON) && (mod_id == MOD_CSTRIKE) && (score.bomb_time < gpGlobals->time))
		{
			score.bomb_time = gpGlobals->time + 1.0;
			
			--score.mp_c4timer;
			
			int freq = (int)sm_bombfreq->value;
			
			if (!freq) freq = 1;
			
			if ((score.mp_c4timer <= sm_bombcont->value) || ((score.mp_c4timer % freq) == 0))
			{
				char time_in_word[64];
				UTIL_IntToString(score.mp_c4timer, time_in_word);
				score.mp_c4word = time_in_word;
				
				//spk \'vox/%s\' or spk \'vox/%s seconds until explosion\'
				char* say_time_left = UTIL_VarArgs("%s\n",
					UTIL_FillWithInfo((score.mp_c4timer <= sm_bombcont->value) ? 
					sm_bombtimecont->string :  sm_bombtimefreq->string));
				
				SERVER_COMMAND(say_time_left);
				
			}
		} 
		
		if (score.stats_time > 0 && score.stats_time < gpGlobals->time)
		{
			score.stats_time = 0.0;
			char pList[1024];
			player_t *pPlayer;
			
			for (int a = 0; a <= gpGlobals->maxClients; ++a)
			{
				pPlayer = GET_PLAYER_POINTER_I(a);
				
				if (pPlayer->valid)	{
					MakeStats(pPlayer,pList);
					UTIL_ShowMOTD(pPlayer->pEdict,pList, STRING( pPlayer->pEdict->v.netname ));	
				}
			}
		}	
	}

	RETURN_META(MRES_IGNORED);
}

void PlayerPreThink_Post( edict_t *pEntity ) 
{
	if (sm_enabled)
	{	
		static int index; index = ENTINDEX(pEntity);
		static player_t *pPlayer; pPlayer = GET_PLAYER_POINTER_I(index);
		
		if (pPlayer->freqtime > 0 && SM_MODE(MONSTER_ON) && pPlayer->freqtime < gpGlobals->time)
		{
			pPlayer->freqtime = 0;
			
			if (score.freqkill[index] >= sm_monsterkillnum->value)
			{
				score.kills = score.freqkill[index];
				score.heads = score.freqhs[index];
				score.killer = pEntity->v.netname;
				
				DO_THE_ACTION(sm_monsterkill->string);
			}
			
			score.freqkill[index] = 0;
			score.freqhs[index] = 0;
		}	
	}

	RETURN_META(MRES_IGNORED);
}

void ClientDisconnect_Post( edict_t *pEntity ) 
{
	player_t *pPlayer	= GET_PLAYER_POINTER(pEntity);

	pPlayer->valid = false;

	//Log stats of player who is leaving the server
	if (sm_enabled)
	{
		//Go through weapon array and log fired weapons
		for (int i = 0; weapon_select[i].iId ; ++i)
		{
			if (pPlayer->a_weapons[weapon_select[i].iId].bulletsfired) 
			{
				//name<userid><wonid><team> "weapon x" "shots x" "hits x" "headshots x"
				UTIL_LogPrintf("\"%s<%i><%u><%s>\" triggered \"weaponstats\" %s\n", /*Plugin_info.logtag,*/
					STRING( pEntity->v.netname ), 
					GETPLAYERUSERID( pEntity ),
					GETPLAYERWONID( pEntity ),
					pPlayer->team_name,	
					UTIL_FillWithWeapon(sm_logformat->string,
					weapon_select[i].name,pPlayer->a_weapons[weapon_select[i].iId]));
			}
		}
	}

	RETURN_META(MRES_IGNORED);
}

void ClientPutInServer_Post( edict_t *pEntity ) {

	player_t *pPlayer = GET_PLAYER_POINTER(pEntity);

	memset(pPlayer->a_weapons,0,sizeof(pPlayer->a_weapons));

	//Save his edict
	pPlayer->pEdict			= pEntity;		
	//No last weapon
	pPlayer->kills			= 0;

	pPlayer->deaths			= 0;
	//Reset stats
	pPlayer->freqtime		= 0;

	pPlayer->current_weapon	= 0;

	pPlayer->valid			= (pEntity->v.flags & FL_FAKECLIENT) ? false : true;

	RETURN_META(MRES_IGNORED);
}

void ClientCommand_Post( edict_t *pEntity )
{
	if (sm_enabled)
	{	
		player_t *pPlayer = GET_PLAYER_POINTER(pEntity);
		
		if (SM_MODE(STATS_ON)&&FStrEq(CMD_ARGV(0),"say")&&FStrEq(CMD_ARGV(1),"/statsme"))
		{
			char pList[1024]; 
			MakeStats(pPlayer,pList);
			UTIL_ShowMOTD(pEntity,pList, STRING( pEntity->v.netname ));
		}
		
	}

	RETURN_META(MRES_IGNORED);
}

// from SDK dlls/cbase.cpp:
// (wd) indicates my comments on the functions
static DLL_FUNCTIONS gFunctionTable_Post = {
	NULL,//! pfnGameInit()				Initialize the game (one-time call after loading of game .dll)
	NULL,//! pfnSpawn()
	NULL,//DispatchThink,				//! pfnThink()
	NULL,//DispatchUse,				//! pfnUse()
	NULL,//DispatchTouch,				//! pfnTouch()
	NULL,//DispatchBlocked,			//! pfnBlocked()
	NULL,//DispatchKeyValue,			//! pfnKeyValue()
	NULL,//DispatchSave,				//! pfnSave()
	NULL,//DispatchRestore,			//! pfnRestore()
	NULL,//DispatchObjectCollsionBox,	//! pfnSetAbsBox()

	NULL,//SaveWriteFields,			//! pfnSaveWriteFields()
	NULL,//SaveReadFields,				//! pfnSaveReadFields()

	NULL,//SaveGlobalState,			//! pfnSaveGlobalState()
	NULL,//RestoreGlobalState,			//! pfnRestoreGlobalState()
	NULL,//ResetGlobalState,			//! pfnResetGlobalState()

	NULL,//! pfnClientConnect()			(wd) Client has connected
	ClientDisconnect_Post,			//! pfnClientDisconnect()		(wd) Player has left the game
	NULL,//ClientKill,					//! pfnClientKill()				(wd) Player has typed "kill"
	ClientPutInServer_Post,			//! pfnClientPutInServer()		(wd) Client is entering the game
	ClientCommand_Post,//! pfnClientCommand()			(wd) Player has sent a command (typed, or from a bind)
	NULL,//! pfnClientUserInfoChanged()	(wd) Client has updated their setinfo structure
	ServerActivate_Post,//ServerActivate,				//! pfnServerActivate()			(wd) Server is starting a new map
	NULL,//ServerDeactivate,			//! pfnServerDeactivate()		(wd) Server is leaving the map (shutdown, or changelevel); SDK2

	PlayerPreThink_Post,//PlayerPreThink,				//! pfnPlayerPreThink()
	NULL,//PlayerPostThink,			//! pfnPlayerPostThink()

	StartFrame_Post,//! pfnStartFrame()
	NULL,//ParmsNewLevel,				//! pfnParmsNewLevel()
	NULL,//ParmsChangeLevel,			//! pfnParmsChangeLevel()

	NULL,//GetGameDescription,			//! pfnGetGameDescription()		Returns string describing current .dll.  E.g. "TeamFotrress 2", "Half-Life"
	NULL,//PlayerCustomization,		//! pfnPlayerCustomization()	Notifies .dll of new customization for player.

	NULL,//SpectatorConnect,			//! pfnSpectatorConnect()		Called when spectator joins server
	NULL,//SpectatorDisconnect,		//! pfnSpectatorDisconnect()	Called when spectator leaves the server
	NULL,//SpectatorThink,				//! pfnSpectatorThink()			Called when spectator sends a command packet (usercmd_t)
	
	NULL,//Sys_Error,					//! pfnSys_Error()				Notify game .dll that engine is going to shut down.  Allows mod authors to set a breakpoint.  SDK2

	NULL,//PM_Move,					//! pfnPM_Move()				(wd) SDK2
	NULL,//PM_Init,					//! pfnPM_Init()				Server version of player movement initialization; (wd) SDK2
	NULL,//PM_FindTextureType,			//! pfnPM_FindTextureType()		(wd) SDK2
	
	NULL,//SetupVisibility,			//! pfnSetupVisibility()		Set up PVS and PAS for networking for this client; (wd) SDK2
	NULL,//UpdateClientData,			//! pfnUpdateClientData()		Set up data sent only to specific client; (wd) SDK2
	NULL,//AddToFullPack,				//! pfnAddToFullPack()			(wd) SDK2
	NULL,//CreateBaseline,				//! pfnCreateBaseline()			Tweak entity baseline for network encoding, allows setup of player baselines, too.; (wd) SDK2
	NULL,//RegisterEncoders,			//! pfnRegisterEncoders()		Callbacks for network encoding; (wd) SDK2
	NULL,//GetWeaponData,				//! pfnGetWeaponData()			(wd) SDK2
	NULL,//CmdStart,					//! pfnCmdStart()				(wd) SDK2
	NULL,//CmdEnd,						//! pfnCmdEnd()					(wd) SDK2
	NULL,//ConnectionlessPacket,		//! pfnConnectionlessPacket()	(wd) SDK2
	NULL,//GetHullBounds,				//! pfnGetHullBounds()			(wd) SDK2
	NULL,//CreateInstancedBaselines,	//! pfnCreateInstancedBaselines()	(wd) SDK2
	NULL,//InconsistentFile,			//! pfnInconsistentFile()		(wd) SDK2
	NULL,//AllowLagCompensation,		//! pfnAllowLagCompensation()	(wd) SDK2
};

C_DLLEXPORT int GetEntityAPI2_Post( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion )
{
	LOG_DEVELOPER(PLID, "called: GetEntityAPI2_Post; version=%d", *interfaceVersion);
	if(!pFunctionTable) {
		LOG_ERROR(PLID, "GetEntityAPI2_Post called with null pFunctionTable");
		return(FALSE);
	}
	else if(*interfaceVersion != INTERFACE_VERSION) {
		LOG_ERROR(PLID, "GetEntityAPI2_Post version mismatch; requested=%d ours=%d", *interfaceVersion, INTERFACE_VERSION);
		//! Tell engine what version we had, so it can figure out who is out of date.
		*interfaceVersion = INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy( pFunctionTable, &gFunctionTable_Post, sizeof( DLL_FUNCTIONS ) );
	return(TRUE);
}

// Must provide at least one of these..
META_FUNCTIONS gMetaFunctionTable = {
	NULL,					// pfnGetEntityAPI				HL SDK; called before game DLL
	NULL,					// pfnGetEntityAPI_Post			META; called after game DLL
	NULL,			// pfnGetEntityAPI2				HL SDK2; called before game DLL
	GetEntityAPI2_Post,		// GetEntityAPI2_Post,			// pfnGetEntityAPI2_Post		META; called after game DLL
	NULL,					// pfnGetNewDLLFunctions		HL SDK2; called before game DLL
	NULL,					// GetNewDLLFunctions_Post,	// pfnGetNewDLLFunctions_Post	META; called after game DLL
	NULL,					// pfnGetEngineFunctions		META; called before HL engine
	GetEngineFunctions_Post	// pfnGetEngineFunctions_Post	META; called after HL engine
};