// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// engine_api.cpp - implementation of Half-Life engine functions

#include <stdio.h>
#include <stdarg.h>

#include <extdll.h>

#include "engine_api.h"
#include "meta_api.h"
#include "statsme_api.h"

// messages created in RegUserMsg which will be "caught"
int	gmsgDeathMsg		= 0;
int	gmsgDamage			= 0;
int	gmsgCurWeapon		= 0;
int	gmsgTextMsg			= 0;
int	gmsgStatusIcon		= 0;
int	gmsgTeamInfo		= 0;
int	gmsgWeaponList		= 0;
int	gmsgResetHUD		= 0;
int	gmsgintermission	= 0;
int	gmsgAmmoX			= 0;

// for positioning values in server messages
int	client_state		= 0;

// player_t of player to who server message goes
player_t *modMsgPlayer	= NULL;

void (*modMsgFunction)(void *, player_t *pPlayer) = NULL;
void (*modMsgEndFunction)(void *, player_t *pPlayer) = NULL;

void Client_DeathMsg(void *p, player_t *pPlayer);
void Client_Damage(void *p, player_t *pPlayer);
void Client_CurWeapon(void *p, player_t *pPlayer);
void Client_TextMsg(void *p, player_t *pPlayer);
void Client_StatusIcon(void *p, player_t *pPlayer);
void Client_TeamInfo(void *p, player_t *pPlayer);
void Client_WeaponList(void *p, player_t *pPlayer);

void MessageBegin_Post(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed) {

	client_state		= 0;
	modMsgFunction		= NULL;     // no msg function until known otherwise
	modMsgEndFunction	= NULL;  // no msg end function until known otherwise
	modMsgPlayer		= NULL;

	if (sm_enabled)
	{

		modMsgPlayer = (ed) ? GET_PLAYER_POINTER(ed) : NULL;

		if ((msg_type == gmsgDamage) && modMsgPlayer)
		{
			modMsgFunction = Client_Damage;
		}
		else if (msg_type == gmsgDeathMsg)
		{
			modMsgFunction = Client_DeathMsg;
		}
		else if ((msg_type == gmsgTextMsg) && (mod_id == MOD_CSTRIKE) && !modMsgPlayer)
		{
			modMsgFunction = Client_TextMsg;
		}
		else if ((msg_type == gmsgStatusIcon) && (mod_id == MOD_CSTRIKE))
		{
			modMsgFunction = Client_StatusIcon;
		}
		else if ((msg_type == gmsgTeamInfo) && !modMsgPlayer)
		{
			modMsgFunction = Client_TeamInfo;	
		}
		else if (msg_type == gmsgResetHUD &&(mod_id == MOD_CSTRIKE)&&(score.mp_c4timer==-1))
		{
			score.mp_c4timer = 0; //Allow for counting down
		}
		else if (msg_type == gmsgCurWeapon)
		{
			modMsgFunction = Client_CurWeapon;
		}
		else if (msg_type == gmsgWeaponList)
		{
			modMsgFunction = Client_WeaponList;
		}
		else if (msg_type == gmsgintermission && SM_MODE(SCORE_ON))
		{
			score.stats_time = gpGlobals->time + 2.0;
		}
	}
	
	RETURN_META(MRES_IGNORED);
}

void MessageEnd_Post(void) {
	if (modMsgEndFunction)(*modMsgEndFunction)(NULL, modMsgPlayer);  // NULL indicated msg end
	modMsgFunction = NULL;
	modMsgEndFunction = NULL;
	RETURN_META(MRES_IGNORED);
}

void WriteByte_Post(int iValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&iValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}

void WriteChar_Post(int iValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&iValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteShort_Post(int iValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&iValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteLong_Post(int iValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&iValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteAngle_Post(float flValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&flValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteCoord_Post(float flValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&flValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteString_Post(const char *sz) {
	if (modMsgFunction)(*modMsgFunction)((void *)sz, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}
void WriteEntity_Post(int iValue) {
	if (modMsgFunction)(*modMsgFunction)((void *)&iValue, modMsgPlayer);
	RETURN_META(MRES_IGNORED);
}

enginefuncs_t meta_engfuncs_post = {
	NULL,			// pfnPrecacheModel()
	NULL,			// pfnPrecacheSound()
	NULL,				// pfnSetModel()
	NULL,				// pfnModelIndex()
	NULL,			// pfnModelFrames()

	NULL,				// pfnSetSize()
	NULL,			// pfnChangeLevel()
	NULL,			// pfnGetSpawnParms()
	NULL,			// pfnSaveSpawnParms()

	NULL,				// pfnVecToYaw()
	NULL,			// pfnVecToAngles()
	NULL,			// pfnMoveToOrigin()
	NULL,				// pfnChangeYaw()
	NULL,			// pfnChangePitch()

	NULL,		// pfnFindEntityByString()
	NULL,			// pfnGetEntityIllum()
	NULL,		// pfnFindEntityInSphere()
	NULL,		// pfnFindClientInPVS()
	NULL,			// pfnEntitiesInPVS()

	NULL,			// pfnMakeVectors()
	NULL,			// pfnAngleVectors()

	NULL,			// pfnCreateEntity()
	NULL,			// pfnRemoveEntity()
	NULL,		// pfnCreateNamedEntity()

	NULL,				// pfnMakeStatic()
	NULL,			// pfnEntIsOnFloor()
	NULL,			// pfnDropToFloor()

	NULL,				// pfnWalkMove()
	NULL,				// pfnSetOrigin()

	NULL,				// pfnEmitSound()
	NULL,		// pfnEmitAmbientSound()

	NULL,				// pfnTraceLine()
	NULL,				// pfnTraceToss()
	NULL,		// pfnTraceMonsterHull()
	NULL,				// pfnTraceHull()
	NULL,				// pfnTraceModel()
	NULL,			// pfnTraceTexture()
	NULL,			// pfnTraceSphere()
	NULL,			// pfnGetAimVector()

	NULL,			// pfnServerCommand()
	NULL,			// pfnServerExecute()
	NULL,		// pfnClientCommand()	// d'oh, ClientCommand in dllapi too (eng)

	NULL,			// pfnParticleEffect()
	NULL,				// pfnLightStyle()
	NULL,				// pfnDecalIndex()
	NULL,			// pfnPointContents()

	MessageBegin_Post,			// pfnMessageBegin()
	MessageEnd_Post,				// pfnMessageEnd()

	WriteByte_Post,				// pfnWriteByte()
	WriteChar_Post,				// pfnWriteChar()
	WriteShort_Post,				// pfnWriteShort()
	WriteLong_Post,				// pfnWriteLong()
	WriteAngle_Post,				// pfnWriteAngle()
	WriteCoord_Post,				// pfnWriteCoord()
	WriteString_Post,			// pfnWriteString()
	WriteEntity_Post,			// pfnWriteEntity()

	NULL,			// pfnCVarRegister()
	NULL,			// pfnCVarGetFloat()
	NULL,			// pfnCVarGetString()
	NULL,			// pfnCVarSetFloat()
	NULL,			// pfnCVarSetString()

	NULL,			// pfnAlertMessage()
	NULL,			// pfnEngineFprintf()

	NULL,	// pfnPvAllocEntPrivateData()
	NULL,		// pfnPvEntPrivateData()
	NULL,		// pfnFreeEntPrivateData()

	NULL,			// pfnSzFromIndex()
	NULL,			// pfnAllocString()

	NULL, 			// pfnGetVarsOfEnt()
	NULL,		// pfnPEntityOfEntOffset()
	NULL,		// pfnEntOffsetOfPEntity()
	NULL,			// pfnIndexOfEdict()
	NULL,		// pfnPEntityOfEntIndex()
	NULL,		// pfnFindEntityByVars()
	NULL,			// pfnGetModelPtr()

	NULL,				// pfnRegUserMsg()

	NULL,		// pfnAnimationAutomove()
	NULL,		// pfnGetBonePosition()

	NULL,		// pfnFunctionFromName()
	NULL,		// pfnNameForFunction()

	NULL,			// pfnClientPrintf()			//! JOHN: engine callbacks so game DLL can print messages to individual clients
	NULL,			// pfnServerPrint()

	NULL,				// pfnCmd_Args()				//! these 3 added 
	NULL,				// pfnCmd_Argv()				//! so game DLL can easily 
	NULL,				// pfnCmd_Argc()				//! access client 'cmd' strings

	NULL,			// pfnGetAttachment()

	NULL,				// pfnCRC32_Init()
	NULL,	// pfnCRC32_ProcessBuffer()
	NULL,		// pfnCRC32_ProcessByte()
	NULL,			// pfnCRC32_Final()

	NULL,				// pfnRandomLong()
	NULL,			// pfnRandomFloat()

	NULL,				// pfnSetView()
	NULL,					// pfnTime()
	NULL,			// pfnCrosshairAngle()

	NULL,			// pfnLoadFileForMe()
	NULL,				// pfnFreeFile()

	NULL,				// pfnEndSection()				//! trigger_endsection
	NULL,		// pfnCompareFileTime()
	NULL,				// pfnGetGameDir()
	NULL,	// pfnCvar_RegisterVariable()
	NULL,		// pfnFadeClientVolume()
	NULL,		// pfnSetClientMaxspeed()
	NULL,		// pfnCreateFakeClient() 		//! returns NULL if fake client can't be created
	NULL,			// pfnRunPlayerMove()
	NULL,		// pfnNumberOfEntities()

	NULL,		// pfnGetInfoKeyBuffer()		//! passing in NULL gets the serverinfo
	NULL,			// pfnInfoKeyValue()
	NULL,			// pfnSetKeyValue()
	NULL,		// pfnSetClientKeyValue()

	NULL,				// pfnIsMapValid()
	NULL,			// pfnStaticDecal()
	NULL,		// pfnPrecacheGeneric()
	NULL, 		// pfnGetPlayerUserId()			//! returns the server assigned userid for this player.
	NULL,			// pfnBuildSoundMsg()
	NULL,		// pfnIsDedicatedServer()		//! is this a dedicated server?
	NULL,			// pfnCVarGetPointer()
	NULL,			// pfnGetPlayerWONId()			//! returns the server assigned WONid for this player.

	//! YWB 8/1/99 TFF Physics additions
	NULL,			// pfnInfo_RemoveKey()
	NULL,		// pfnGetPhysicsKeyValue()
	NULL,		// pfnSetPhysicsKeyValue()
	NULL,	// pfnGetPhysicsInfoString()
	NULL,			// pfnPrecacheEvent()
	NULL,			// pfnPlaybackEvent()

	NULL,				// pfnSetFatPVS()
	NULL,				// pfnSetFatPAS()

	NULL,		// pfnCheckVisibility()

	NULL,			// pfnDeltaSetField()
	NULL,		// pfnDeltaUnsetField()
	NULL,		// pfnDeltaAddEncoder()
	NULL,		// pfnGetCurrentPlayer()
	NULL,			// pfnCanSkipPlayer()
	NULL,			// pfnDeltaFindField()
	NULL,	// pfnDeltaSetFieldByIndex()
	NULL,	// pfnDeltaUnsetFieldByIndex()

	NULL,			// pfnSetGroupMask()

	NULL, // pfnCreateInstancedBaseline()		// d'oh, CreateInstancedBaseline in dllapi too
	NULL,			// pfnCvar_DirectSet()

	NULL,		// pfnForceUnmodified()

	NULL,			// pfnGetPlayerStats()

	NULL,		// pfnAddServerCommand()

	// Added in SDK 2.2:
	NULL,	// pfnVoice_GetClientListening()
	NULL,	// pfnVoice_SetClientListening()
};

C_DLLEXPORT int GetEngineFunctions_Post(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion ) 
{
	LOG_DEVELOPER(PLID, "called: GetEngineFunctions_Post; version=%d", *interfaceVersion);
	if(!pengfuncsFromEngine) {
		LOG_ERROR(PLID, "GetEngineFunctions_Post called with null pengfuncsFromEngine");
		return(FALSE);
	}
	else if(*interfaceVersion != ENGINE_INTERFACE_VERSION) {
		LOG_ERROR(PLID, "GetEngineFunctions_Post version mismatch; requested=%d ours=%d", *interfaceVersion, ENGINE_INTERFACE_VERSION);
		// Tell metamod what version we had, so it can figure out who is out of date.
		*interfaceVersion = ENGINE_INTERFACE_VERSION;
		return(FALSE);
	}
	memcpy(pengfuncsFromEngine, &meta_engfuncs_post, sizeof(enginefuncs_t));
	return TRUE;
}
