// vi: set ts=4 sw=4 :
// vim: set tw=75 :

// trace_api.h - (main) description of HL API tracing operations

/*
 * Copyright (c) 2001 Will Day <willday@hpgx.net>
 *
 *    This file is part of Metamod.
 *
 *    Metamod is free software; you can redistribute it and/or modify it
 *    under the terms of the GNU General Public License as published by the
 *    Free Software Foundation; either version 2 of the License, or (at
 *    your option) any later version.
 *
 *    Metamod is distributed in the hope that it will be useful, but
 *    WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Metamod; if not, write to the Free Software Foundation,
 *    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    In addition, as a special exception, the author gives permission to
 *    link the code of this program with the Half-Life Game Engine ("HL
 *    Engine") and Modified Game Libraries ("MODs") developed by Valve,
 *    L.L.C ("Valve").  You must obey the GNU General Public License in all
 *    respects for all of the code used other than the HL Engine and MODs
 *    from Valve.  If you modify this file, you may extend this exception
 *    to your version of the file, but you are not obligated to do so.  If
 *    you do not wish to do so, delete this exception statement from your
 *    version.
 *
 */

#ifndef TRACE_API_H
#define TRACE_API_H

#include <time.h>				// time()

#include <enginecallback.h>		// ALERT()
#include <sdk_util.h>			// UTIL_VarArgs()
#include <api_info.h>

// a = shots, b = hits
#define ACCURACY_FORMULA(a,b) ((a) ? ( (float)b * 100.00 / (float)a ) : 0.00 )

// a = kills, b = deaths
#define EFFEC_FORMULA(a,b) ((b + a) ? ( (float)a * 100.00 / (float)( b + a ) ) : 0.00 )

#define	MOD_UNKNOW		0
#define	MOD_CSTRIKE		1
#define	MOD_TFC			2
#define	MOD_FLF			3
#define	MOD_DMC			4
#define	MOD_DOD			5
#define	MOD_FIREARMS	6
#define	MOD_ACTION		7
#define	MOD_VALVE		8

extern short int	mod_id;
//extern int			weapon_list; //Keeps bits of know weapons

struct announce_t
{
	string_t	bombplanter;
	float		bomb_time;
	short int	mp_c4timer;
	const char* mp_c4word;

	short int	bestkill[33];
	short int   besths[33];
	short int	game_score[3];

	short int	freqkill[33];
	short int   freqhs[33];

	string_t	killer;
	string_t	victim;
	short int	heads;
	short int	kills;
	short int	deaths;
	const char*	weapon;

	float		killstodeaths;
	float		accuracy;

	float		stats_time;
};

typedef struct
{
	short int	iId;				// the weapon ID value
	char		name[16];	// name of the weapon when selecting it

} weapon_select_struct;

//WEAPON STATS
struct weapon_stat_t
{
	int			bulletsfired;
	int			bulletshit;
	int			lastammo;
	int			headshot;
	int			kills;
	int			tk;
	int			health;
	int			deaths;

};

struct player_t
{

	edict_t*				pEdict;

	float					freqtime;
	
	char					team_name[64];

	short int				deaths;
	short int				kills;

	bool					valid;

	int						current_weapon;

	struct weapon_stat_t	a_weapons[32];

};

#define GET_PLAYER_POINTER(e) &players[ENTINDEX(e)]
#define GET_PLAYER_POINTER_I(i) &players[i]

extern int	gmsgDeathMsg;
extern int	gmsgDamage;
extern int	gmsgCurWeapon;
extern int	gmsgTextMsg;
extern int	gmsgStatusIcon;
extern int	gmsgTeamInfo;
extern int	gmsgWeaponList;
extern int	gmsgResetHUD;
extern int	gmsgMOTD;
extern int	gmsgServerName;
extern int	gmsgintermission;
extern int	gmsgCurWeapon;
extern int	gmsgAmmoX;

extern announce_t				score;
extern player_t					players[33];
extern weapon_select_struct		weapon_select[MAX_WEAPONS+1];

extern int		client_state;

extern cvar_t	*statsme_version;
extern cvar_t	*sm_logformat;

extern cvar_t	*sm_headshot;
extern cvar_t	*sm_bombset;
extern cvar_t	*sm_bombtime;
extern cvar_t	*sm_kill;
extern cvar_t	*sm_monsterkill;
extern cvar_t	*sm_monsterkillnum;
extern cvar_t	*sm_roundend;

extern cvar_t	*sm_bombtimefreq;
extern cvar_t	*sm_bombtimecont;
extern cvar_t	*sm_bombfreq;
extern cvar_t	*sm_bombcont;

extern cvar_t	*sm_playerstats;
extern cvar_t	*sm_weaponstats;

extern cvar_t	*sm_mode;

extern cvar_t	*sm_monsterfreq;

extern bool		sm_enabled;


// SM MODE
inline bool SM_MODE(int a) {
	return (((int)sm_mode->value & a)!=0);
}


#define	STATS_ON	1 //allow for displaying stats during the game (say /statsme)
#define SCORE_ON	2 //show stats to all players at the end of map
#define KILL_ON		4 //announce normal kill
#define HEAD_ON		8 //announce headshot
#define MONSTER_ON	16 //announce monsterkill
#define BOMB_ON		32 //announce bomb set (only CS)
#define TIME_ON		64 //counting down to explosion (only CS)
#define ROUND_ON	128 //round end score (only CS)

char	*UTIL_FillWithWeapon(const char* log_format, const char* wname, weapon_stat_t& a_weapons);
void	statsme_init(void);

void	UTIL_ShowMOTD( edict_t *client , char *pList, const char *szName);
void	UTIL_HudMessage(const hudtextparms_t &textparms, char *pMessage);

char* UTIL_FillWithInfo(const char* string);

void UTIL_IntToString(int value, char *output);

void CmdSay(void);
void CmdClExec(void);

#define DO_THE_ACTION(a) SERVER_COMMAND(UTIL_VarArgs("%s\n",UTIL_FillWithInfo(a)));

#endif /* TRACE_API_H */
